



##############################
#####  Useful functions  #####
##############################

iPCinOrd <- function (x1, x2) {
  loci=which(x2 %in% x1)
  if (x2[loci[1]]==x1[1]) {
    return(1)
  } else {
    return(0)
  }

}


exPCinOrd <- function (x1, x2) {
  loci=which(x2 %in% x1)
  x2[rev(loci)]=x2[loci]
  return(x2)
}

VtoM2 <- function(sigma2ui, indexGi, pM) {
  indexGi[,3]=1:nrow(indexGi)
  out=matrix(0, pM, pM)
  
  for (s in 1:pM) {
    for (r in 1:pM) {
      indexi=indexGi[,1]==s & indexGi[,2]==r
      if (any(indexi)) {
        indexi=indexGi[indexi,3]
        out[s,r]=sigma2ui[indexi]
      }
    }
  }
  
  return(out)
}

pVtoM2<-function(sigma2u,indexG,pM) {
  tem=VtoM2(round(sigma2u,3),indexG,pM);tem[tem==0]="";tem[,1]="0";tem[tem[,ncol(tem)]=="",ncol(tem)]="0"
  print(tem,quote=F)
}

GenPCm <- function(x) {
  ## Directed pair
  ordDP=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (s != r) {
        if (x[sr]>0) {
          ordDP=rbind(ordDP,c(s=s,r=r))
        }
        sr=sr+1
      }
    }
  }
  nDP=nrow(ordDP)
  
  ## Parent / child matrix
  i=1; mPC=matrix(0, ncol=pM, nrow=pM)
  for (i in 1:nDP) {
    i1=ordDP[i,1]; i2=ordDP[i,2]
    mPC[i1,i2]=1
    
    # child down
    indexi2=which(mPC[i2,]!=0)
    indexi2=unique(c(i2,indexi2))
    # parent up
    indexi1=which(mPC[,i1]!=0)
    indexi1=unique(c(i1,indexi1))
    
    # assign parents & children
    mPC[indexi1,indexi2]=1
  }
  
  # mPC2=matrix(0, ncol=pM, nrow=pM)
  # for (i in 1:nrow(ordDP)) {
  #   i1=ordDP[i,1]; i2=ordDP[i,2]
  #   i1; i2
  #   mPC2[i1,i2]=1
  #   ## child
  #   if (any(mPC2[i2,]!=0)) {
  #     indexi=which(mPC2[i2,]!=0)
  #     mPC2[i1,indexi]=1
  #   }
  #   ## parent
  #   if (any(mPC2[,i1]!=0)) {
  #     indexi=which(mPC2[,i1]!=0)
  #     mPC2[indexi,i2]=1
  #   }
  # }
  # 
  # all.equal(mPC,mPC2)
  return(mPC)
}

##############################
###  Log-likelihood upper  ###
##############################

upLL1 <- function (M, orderi, theta1, indexij) {
  # orderi=orderS[,i]; sigma2uFj=sigma2uj; sigma2mj=sigma2mi
  
  Mi=M[,orderi]; pM=ncol(M)
  pUF=pM*(pM-1)
  
  theta=numeric(pM+pUF)
  theta[indexij]=theta1
  # theta[-indexij]=0
  
  sigma2mj=theta[1:pM]
  sigma2uFj=theta[-c(1:pM)]
  
  # return(list(s1=sigma2uFj,s2=sigma2mj))
  
  sigma2mi=sigma2mj[orderi]
  
  indexF=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (r != s) {
        indexF=rbind(indexF, c(s=s,r=r,sr=sr))
        sr=sr+1
      }
    }
  }
  
  Bi=VtoM2(sigma2uFj, indexF, pM)
  Bi2=matrix(0, nrow=pM, ncol=pM)
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      # Bi2[orderi[s],orderi[r]]=Bi[s,r]
      Bi2[s,r]=Bi[orderi[s],orderi[r]]
    }
  }
  sigma2ui=numeric(pU); sr=1
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      sigma2ui[sr]=Bi2[s,r]
      sr=sr+1
    }
  }
  sigma2ui
  
  indexG0i=NULL; sr=1; sr2=1
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      if (sigma2ui[sr]>0) {
        indexG0i=rbind(indexG0i, c(s=s,r=r,sr=sr2))
        sr2=sr2+1
      }
      sr=sr+1
    }
  }
  
  # return(mSEMup(Mi, sigma2mi, sigma2ui, indexG0i))
  out=c(mSEMup(Mi, sigma2mi, sigma2ui, indexG0i)/n)
  return(out)
}


upLL <- function (M, orderi, sigma2uFj, sigma2mj) {
  # orderi=orderS[,i]; sigma2uFj=sigma2uj; sigma2mj=sigma2mi
  Mi=M[,orderi]; pM=ncol(M)
  sigma2mi=sigma2mj[orderi]
  
  indexF=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (r != s) {
        indexF=rbind(indexF, c(s=s,r=r,sr=sr))
        sr=sr+1
      }
    }
  }
  
  Bi=VtoM2(sigma2uFj, indexF, pM)
  Bi2=matrix(0, nrow=pM, ncol=pM)
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      # Bi2[orderi[s],orderi[r]]=Bi[s,r]
      Bi2[s,r]=Bi[orderi[s],orderi[r]]
    }
  }
  sigma2ui=numeric(pU); sr=1
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      sigma2ui[sr]=Bi2[s,r]
      sr=sr+1
    }
  }
  sigma2ui
  
  indexG0i=NULL; sr=1; sr2=1
  for (s in 1:(pM-1)) {
    for (r in (s+1):pM) {
      if (sigma2ui[sr]>0) {
        indexG0i=rbind(indexG0i, c(s=s,r=r,sr=sr2))
        sr2=sr2+1
      }
      sr=sr+1
    }
  }
  
  # return(mSEMup(Mi, sigma2mi, sigma2ui, indexG0i))
  out=c(mSEMup(Mi, sigma2mi, sigma2ui, indexG0i)/n)
  return(out)
}


mSEMup <- function(M, sigma2m, sigma2u, indexG0) {
  
  n=nrow(M)
  pU0=sum(sigma2u!=0); pM=length(sigma2m)
  
  sigma2ui=c(sigma2u[sigma2u!=0])
  sigma2mi=sigma2m
  
  if (pU0>1) {
    Sigmai=diag(1/sigma2ui)
  } else {
    Sigmai=1/sigma2ui
  }
  Einv=diag(1/sigma2mi)
  
  ml=0.0
  for (i in 1:n) {
    
    M2=M[i,]%*%t(M[i,])
    
    temW=numeric(pU0)
    for (s in 1:pM) {
      indexi=indexG0[,"s"]==s
      if (any(indexi)) {
        loci=which(indexi); ri=indexG0[indexi,"r"]
        
        temW[loci]=M[i,s]*M[i,ri]/sigma2mi[s]
      }
    }
    C1=temW
    
    # M2=M[i,]%*%t(M[i,])
    temW2=matrix(0, ncol=pU0, nrow=pU0)
    for (s in 1:pM) {
      indexi=indexG0[,"s"]==s
      if (any(indexi)) {
        loci=which(indexi); ri=indexG0[indexi,"r"]
        
        temW2[loci,loci]=M2[ri,ri]/sigma2mi[s]
      }
    }
    
    C2=Sigmai+temW2
    
    
    covi=solve(C2)
    mui=covi%*%C1
    
    ml=ml+t(M[i,])%*%Einv%*%M[i,]-t(C1)%*%covi%*%C1+sum(log(sigma2mi))+sum(log(sigma2ui))+unlist(determinant(C2, logarithm = TRUE))[1]
    
    # return(list(C1=C1,C2=C2,covi=covi,ml=ml))
  }
  return(ml)
}



###########################
###  Get order for one  ###
###########################
GetOrd1 <- function (sigma2ui, pM) {
  # sigma2mi=fitSi$sigma2m[,2]; sigma2ui=fitSi$sigma2u[,2]
  
  nzerou=sum(sigma2ui!=0)
  orderu=order(sigma2ui, decreasing=TRUE)
  
  ## All paired
  indexF=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (r != s) {
        indexF=rbind(indexF, c(s=s,r=r,sr=sr))
        sr=sr+1
      }
    }
  }
  
  ## In-order
  orderPC=NULL
  if (nzerou > 1) {
    for (i in 1:nzerou) {
      orderPC=rbind(orderPC, indexF[orderu[i],1:3])
    }
  } else {
    orderPC=matrix(indexF[orderu[i],1:3], nrow=1)
  }
  
  ## General parent/child; check DAG order
  i=1; orderi=1:pM; indexPC=matrix(0, ncol=pM, nrow=pM); indexPCS=list(); orderS=NULL
  for (i in 1:nrow(orderPC)) {
    i1=orderPC[i,1]; i2=orderPC[i,2]
    
    indexPC[i1,i2]=1
    ## child
    if (any(indexPC[i2,]!=0)) {
      indexi=which(indexPC[i2,]!=0)
      indexPC[i1,indexi]=1
    }
    ## parent
    if (any(indexPC[,i1]!=0)) {
      indexi=which(indexPC[,i1]!=0)
      indexPC[indexi,i2]=1
    }
    indexPCS[[i]]=indexPC
    
    if (any(diag(indexPC)!=0)) {
      indexPCS=indexPCS[-i]
      break
    }
    
    ## exchange order
    if (iPCinOrd(orderPC[i,1:2],orderi)) {
      orderS=cbind(orderS, orderi)
    } else {
      orderi=exPCinOrd(orderPC[i,1:2],orderi)
      repeat {
        ij=0
        for (j in 1:i) {
          if (iPCinOrd(orderPC[j,1:2],orderi)==0) {
            orderi=exPCinOrd(orderPC[j,1:2],orderi)
            ij=1
          }
        }
        if (ij==0) break
      }
      
      orderS=cbind(orderS, orderi)
    }
  }
  
  if (ncol(orderS)==1) {
    orderPC=matrix(orderPC[1,],nrow=1)
  }
  return(list(orderS=orderS, orderPC=orderPC, orderu=orderu))
}



################################
###  Get order for multiple  ###
################################
GetOrdm <- function (sigma2ui, pM) {
  # sigma2ui=fitSi$sigma2u
  # orderu
  nzerou=ncol(sigma2ui)
  
  ## All paired
  indexF=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (r != s) {
        indexF=rbind(indexF, c(s=s,r=r,sr=sr))
        sr=sr+1
      }
    }
  }
  
  ## In-order
  orderPC=NULL
  for (i in nzerou:3) {
    orderPC=rbind(orderPC, indexF[which(sigma2ui[,i]==0 & sigma2ui[,i-1]!=0),])
  }
  
  ## General parent/child; check DAG order
  i=1; orderi=1:pM; indexPC=matrix(0, ncol=pM, nrow=pM); indexPCS=list(); orderS=NULL
  for (i in 1:nrow(orderPC)) {
    i1=orderPC[i,1]; i2=orderPC[i,2]
    
    indexPC[i1,i2]=1
    ## child
    if (any(indexPC[i2,]!=0)) {
      indexi=which(indexPC[i2,]!=0)
      indexPC[i1,indexi]=1
    }
    ## parent
    if (any(indexPC[,i1]!=0)) {
      indexi=which(indexPC[,i1]!=0)
      indexPC[indexi,i2]=1
    }
    indexPCS[[i]]=indexPC
    
    if (any(diag(indexPC)!=0)) {
      indexPCS=indexPCS[-i]
      break
    }
    
    ## exchange order
    if (iPCinOrd(orderPC[i,1:2],orderi)) {
      orderS=cbind(orderS, orderi)
    } else {
      orderi=exPCinOrd(orderPC[i,1:2],orderi)
      repeat {
        ij=0
        for (j in 1:i) {
          if (iPCinOrd(orderPC[j,1:2],orderi)==0) {
            orderi=exPCinOrd(orderPC[j,1:2],orderi)
            ij=1
          }
        }
        if (ij==0) break
      }
      
      orderS=cbind(orderS, orderi)
    }
  }
  
  if (ncol(orderS)==1) {
    orderPC=matrix(orderPC[1,],nrow=1)
  }
  return(list(orderS=orderS, orderPC=orderPC))
}





######################
#####  Estimate  #####
######################

SEMre <- function(M, nTrim, iniSigma2m, iniSigma2u, thresh=1e-3, maxit=100) {
  # nTrim=pU; iniSigma2m=rep(1,pM); iniSigma2u=rep(1,pUF); thresh=1e-3; maxit=100
  pM=ncol(M); pUF=pM*(pM-1)
  
  if (nTrim > 1) {nTrim=pUF}
  
  ti=proc.time()
  fitSi=SEMreC3(nTrim, M, thresh, maxit, iniSigma2m, iniSigma2u)
  ti=proc.time()-ti
  
  if (nTrim == 1) {
    sigma2mi=fitSi$sigma2m[,2]
    sigma2ui=fitSi$sigma2u[,2]
    
    fitOrdi=GetOrd1(sigma2ui,pM)
    orderu=fitOrdi$orderu; orderS=fitOrdi$orderS; orderPC=fitOrdi$orderPC
    nu=nrow(orderPC)
    
    ## Upper LL
    sigma2uj=numeric(pUF)
    
    i=1; upLLi=NULL; BICi=NULL; AICi=NULL
    for (i in 1:nu) {
      sigma2uj[orderu[i]]=sigma2ui[orderu[i]]
      temi=upLL(M, orderS[,i], sigma2uj, sigma2mi)
      # upLLi=c(temi, upLLi)
      # AICi=c(2*temi*n+2*(sum(sigma2uj!=0)+pM), AICi)
      # BICi=c(2*temi*n+log(n)*(sum(sigma2uj!=0)+pM), BICi)
      upLLi=c(upLLi, temi)
      AICi=c(AICi, 2*temi*n+2*(i+pM))
      BICi=c(BICi, 2*temi*n+log(n)*(i+pM))
    }
    
    il1=which.min(upLLi); il2=which.min(AICi); il3=which.min(BICi);
    
    ## update at the optimal graph
    iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
    iniSigma2u[orderPC[1:il1,3]]=1
    fitSi1=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    
    if (il2 == il1) {
      fitSi2=fitSi1
    } else {
      iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
      iniSigma2u[orderPC[1:il2,3]]=1
      fitSi2=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    }
    
    if (il3 == il2) {
      fitSi3=fitSi2
    } else {
      iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
      iniSigma2u[orderPC[1:il3,3]]=1
      fitSi3=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    }
    
    out=cbind(c(fitSi1$sigma2m,fitSi1$sigma2u),
              c(fitSi2$sigma2m,fitSi2$sigma2u),
              c(fitSi3$sigma2m,fitSi3$sigma2u))
    
  } else if (nTrim>1) {
    sigma2mi=fitSi$sigma2m
    sigma2ui=fitSi$sigma2u
    
    fitOrdi=GetOrdm(sigma2ui,pM)
    orderS=fitOrdi$orderS; orderPC=fitOrdi$orderPC
    nu=nrow(orderPC)
    
    ## Upper LL
    i=1; upLLi=NULL; BICi=NULL; AICi=NULL
    for (i in 1:nu) {
      temi=upLL(M, orderS[,i], sigma2ui[,pU+2-i], sigma2mi[,pU+2-i])
      # upLLi=c(temi, upLLi)
      # AICi=c(2*temi*n+2*(sum(sigma2uj!=0)+pM), AICi)
      # BICi=c(2*temi*n+log(n)*(sum(sigma2uj!=0)+pM), BICi)
      upLLi=c(upLLi, temi)
      AICi=c(AICi, 2*temi*n+2*(sum(sigma2ui[,pU+2-i]!=0)+pM))
      BICi=c(BICi, 2*temi*n+log(n)*(sum(sigma2ui[,pU+2-i]!=0)+pM))
    }
    
    il1=which.min(upLLi); il2=which.min(AICi); il3=which.min(BICi);
    
    ## output
    out=cbind(c(sigma2mi[,pU+2-il1], sigma2ui[,pU+2-il1]),
              c(sigma2mi[,pU+2-il2], sigma2ui[,pU+2-il2]),
              c(sigma2mi[,pU+2-il3], sigma2ui[,pU+2-il3]))
    
  } else {
    out=cbind(c(fitSi$sigma2m,fitSi$sigma2u))
  }
  
  return(list(est=out,time=ti))
}



#########################
###  Estimate - Test  ###
#########################

SEMreT <- function(M, nTrim, iniSigma2m, iniSigma2u, thresh=1e-3, maxit=100) {
  # nTrim=pU; iniSigma2m=rep(1,pM); iniSigma2u=rep(1,pUF); thresh=1e-3; maxit=100
  pM=ncol(M); pUF=pM*(pM-1)
  
  if (nTrim > 1) {nTrim=pUF}
  
  ti=proc.time()
  fitSi=SEMreC3(nTrim, M, thresh, maxit, iniSigma2m, iniSigma2u)
  ti=proc.time()-ti
  
  if (nTrim == 1) {
    sigma2mi=fitSi$sigma2m[,2]
    sigma2ui=fitSi$sigma2u[,2]
    
    fitOrdi=GetOrd1(sigma2ui,pM)
    orderu=fitOrdi$orderu; orderS=fitOrdi$orderS; orderPC=fitOrdi$orderPC
    nu=nrow(orderPC)
    
    ## Upper LL
    sigma2uj=numeric(pUF)
    
    i=1; upLLi=NULL; BICi=NULL; AICi=NULL
    for (i in 1:nu) {
      sigma2uj[orderu[i]]=sigma2ui[orderu[i]]
      temi=upLL(M, orderS[,i], sigma2uj, sigma2mi)
      # upLLi=c(temi, upLLi)
      # AICi=c(2*temi*n+2*(sum(sigma2uj!=0)+pM), AICi)
      # BICi=c(2*temi*n+log(n)*(sum(sigma2uj!=0)+pM), BICi)
      upLLi=c(upLLi, temi)
      AICi=c(AICi, 2*temi*n+2*(i+pM))
      BICi=c(BICi, 2*temi*n+log(n)*(i+pM))
    }
    
    il1=which.min(upLLi); il2=which.min(AICi); il3=which.min(BICi);
    
    ## update at the optimal graph
    iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
    iniSigma2u[orderPC[1:il1,3]]=1
    fitSi1=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    
    if (il2 == il1) {
      fitSi2=fitSi1
    } else {
      iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
      iniSigma2u[orderPC[1:il2,3]]=1
      fitSi2=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    }
    
    if (il3 == il2) {
      fitSi3=fitSi2
    } else {
      iniSigma2m=rep(1,pM); iniSigma2u=rep(0,pUF)
      iniSigma2u[orderPC[1:il3,3]]=1
      fitSi3=SEMreC3(0, M, thresh, maxit, iniSigma2m, iniSigma2u)
    }
    
    out=cbind(c(fitSi1$sigma2m,fitSi1$sigma2u),
              c(fitSi2$sigma2m,fitSi2$sigma2u),
              c(fitSi3$sigma2m,fitSi3$sigma2u))
    
  } else if (nTrim>1) {
    sigma2mi=fitSi$sigma2m
    sigma2ui=fitSi$sigma2u
    
    fitOrdi=GetOrdm(sigma2ui,pM)
    orderS=fitOrdi$orderS; orderPC=fitOrdi$orderPC
    nu=nrow(orderPC)
    
    ## Upper LL
    i=1; upLLi=NULL; BICi=NULL; AICi=NULL
    for (i in 1:nu) {
      temi=upLL(M, orderS[,i], sigma2ui[,pU+2-i], sigma2mi[,pU+2-i])
      # upLLi=c(temi, upLLi)
      # AICi=c(2*temi*n+2*(sum(sigma2uj!=0)+pM), AICi)
      # BICi=c(2*temi*n+log(n)*(sum(sigma2uj!=0)+pM), BICi)
      upLLi=c(upLLi, temi)
      AICi=c(AICi, 2*temi*n+2*(sum(sigma2ui[,pU+2-i]!=0)+pM))
      BICi=c(BICi, 2*temi*n+log(n)*(sum(sigma2ui[,pU+2-i]!=0)+pM))
    }
    
    il1=which.min(upLLi); il2=which.min(AICi); il3=which.min(BICi);
    
    ## output
    out=cbind(c(sigma2mi[,pU+2-il1], sigma2ui[,pU+2-il1]),
              c(sigma2mi[,pU+2-il2], sigma2ui[,pU+2-il2]),
              c(sigma2mi[,pU+2-il3], sigma2ui[,pU+2-il3]))
    
  } else {
    out=cbind(c(fitSi$sigma2m,fitSi$sigma2u))
  }
  
  return(list(est=out,time=ti))
}




