

########################################
#####  Data Generation Parameters  #####
########################################

library(MASS)


##########################################################
###  Gen 4:SEM mixed effect - order & zero + last 0/1  ###
##########################################################

GenDataSEM4 <- function (n, BetaXF, indexB01, sigma2uF, indexU01, sigma2m) {
  
  pM=length(sigma2m); pUF=length(sigma2uF); pX=nrow(BetaXF)
  pu1=length(indexU01); px1=length(indexB01)
  
  ## Standardization - X
  X=cbind(matrix(rnorm((pX-1)*n, 0,1),byrow=TRUE,ncol=pX-1),rbinom(n,1,0.5))
  
  mX=apply(X,2,mean)
  sdX=apply(X,2,sd)
  X=t((t(X)-mX)/sdX)
  
  ## Standardization - U
  U=NULL
  if (pu1 > 0) {
    for (j in 1:pu1) {
      temi=rnorm(n, mean=0, sd=sqrt(sigma2uF[indexU01[j]]))
      temi=(temi-mean(temi))/sd(temi)*sqrt(sigma2uF[indexU01[j]])
      U=cbind(U, temi)
    }
  }
  
  ## M
  sigma2md=diag(sigma2m,nrow=pM,ncol=pM)
  M=NULL; i=1
  for (i in 1:n) {
    
    Bi2=numeric(pUF)
    if (px1 > 0) {
      for (j in 1:px1) {
        Bi2[indexB01[j]]=-sum(X[i,]*BetaXF[,indexB01[j]])
      }
    }
    if (pu1 > 0) {
      for (j in 1:pu1) {
        Bi2[indexU01[j]]=Bi2[indexU01[j]]-U[i,j]
      }
    }
    
    Bi=diag(x=NA,nrow=pM,ncol=pM)
    Bi[!is.na(Bi)]=Bi2
    Bi=t(Bi);diag(Bi)=1.0
    
    mi=c(solve(Bi)%*%mvrnorm(1, mu=numeric(pM), sigma2md))
    
    # sourceCpp("invC.cpp")
    # set.seed(123)
    # mi2=c(invC(Bi)%*%mvrnorm(1, mu=numeric(pM), sigma2md))
    # range(mi-mi2)
    
    ### Record
    M=rbind(M, mi)
  }
  # solve(Bi)%*%diag(sigma2m,nrow=pM,ncol=pM)%*%t(solve(Bi))
  
  return(list(M=M, X=X))
}





############################
###  Gen 5:SEM constant  ###
############################

GenDataSEM5 <- function (n, BetaXF, indexB01, sigma2uF, indexU01, sigma2m) {
  
  pM=length(sigma2m); pUF=length(sigma2uF); pX=nrow(BetaXF)
  pu1=length(indexU01); px1=length(indexB01)
  
  ## Standardization - X
  X=matrix(1, nrow=n, ncol=pX)
  
  Bi2=numeric(pUF)
  if (px1 > 0) {
    for (j in 1:px1) {
      Bi2[indexB01[j]]=-sum(BetaXF[,indexB01[j]])
    }
  }
  Bi=diag(x=NA,nrow=pM,ncol=pM)
  Bi[!is.na(Bi)]=Bi2
  Bi=t(Bi);diag(Bi)=1.0
  
  Binv=solve(Bi)
  
  ## M
  sigma2md=diag(sigma2m,nrow=pM,ncol=pM)
  M=NULL; i=1
  for (i in 1:n) {
    
    mi=c(Binv%*%mvrnorm(1, mu=numeric(pM), sigma2md))

    ### Record
    M=rbind(M, mi)
  }
  # solve(Bi)%*%diag(sigma2m,nrow=pM,ncol=pM)%*%t(solve(Bi))
  
  return(list(M=M, X=X))
}




