

rm(list=ls(all=TRUE))

##############################
#####  Mixed Effect SEM  #####
##############################

library(Rcpp)



#################################
#####  Aim 1: Simulation 2  #####
#################################

source("FunR/Generate_170420.R")
source("FunR/FunS7.R")

library(pcalg)
library(PenPC)

source("FunLM/net_lm2.R")
sourceCpp("FunLM/ADMMnet2.cpp")
sourceCpp("TestRcpp_170228b.cpp")

options(warn=2)


###########################################
###  Sim 1-5:Fixed & Random & constant  ###
###########################################

Ns=c(200,500,1000); pMs=c(20,50,100); 

indexG00=matrix(c(1,3,1,2,2,3,1,4,4,5,6,7,7,8,8,9,8,10,11,12,12,13,14,15),byrow=TRUE,ncol=2)


thresh=1e-4; maxit=1e+3; inThresh=1e-3; inMaxit=10
thresh2=1e-3; maxit2=1e+2

rnb0=0.9

nSim=2

Iset=1; timeS=numeric(5)
for (iset in Iset) {
  cat("\n");cat(iset)
  
  resultBS=list(); resultUS=list(); resultPCS=list(); resultMS=list(); resultTS=list()
  timeSi=NULL; resultCase=NULL
  
  iCase=1; iN=1; ipM=1
  for (iN in 1:length(Ns)) {
    for (ipM in 1:length(pMs)) {
      set.seed(123)
      cat("\n");cat(iCase)
      
      ###  Parameters  ###
      n=Ns[iN]; pM=pMs[ipM];
      pU=(pM-1)*pM/2; pUF=pM*(pM-1)
      
      index0=matrix(0,nrow=pM,ncol=pM)
      for (j in 1:nrow(indexG00)) {index0[indexG00[j,1],indexG00[j,2]]=1}
      index0i=index0; diag(index0i)=NA; index0i=c(t(index0i)); index0i=index0i[!is.na(index0i)]
      index01=index0i; index01[index01==1][c(2,4,6,8)]=0
      index02=index0i; index02[index02==1][c(5,7,10,12)]=0
      
      indexB0i=numeric(pUF); indexU0i=numeric(pUF)
      if (iset %in% c(1,3,5)) {indexB0i[index0i==1]=1}
      if (iset %in% c(2,3)) {indexU0i[index0i==1]=1}
      if (iset == 4) {indexB0i[index01==1]=1; indexU0i[index02==1]=1}
      
      if (iset %in% c(1:4)) {pX=3}
      if (iset == 5) {pX=1}
      
      if (iset %in% c(1:4)) {
        BetaXF=matrix(0,nrow=pX,ncol=pUF)
        BetaXF[,indexB0i==1]=seq(0.5,1.5,length=pX)*(-1)^(seq(pX))
      } else if (iset == 5) {
        BetaXF=matrix(0, nrow=pX, ncol=pUF)
        BetaXF[1,indexB0i==1]=rep(-1,1,length=sum(indexB0i==1))*(-1)^(seq(sum(indexB0i==1)))
      }
      
      sigma2uF=rep(0, length=pUF)
      sigma2uF[indexU0i==1]=0.5
      
      sigma2m=rep(c(1,0.5), length=pM)
      
      indexB0=indexB0i
      indexU0=indexU0i
      
      indexB01=which(indexB0!=0)
      indexU01=which(indexU0!=0)
      
      indexF=NULL; sr=1
      for (s in 1:pM) {
        for (r in 1:pM) {
          if (r != s) {
            indexF=rbind(indexF, c(s=s,r=r,sr=sr))
            sr=sr+1
          }
        }
      }
      
      
      ###  Run  ###
      resultBi=list(); resultUi=list(); resultPCi=list(); resultMi=list()
      timeSi=numeric(5); caseSi=0
      
      iSim=1; cat("\n")
      for (iSim in 1:nSim) {
        cat(iSim)
        
        ti=proc.time() #####
        #######################
        ###  Generate Data  ###
        #######################
        if (iset %in% c(1:4)) {
          tmpD=GenDataSEM4(n, BetaXF, indexB01, sigma2uF, indexU01, sigma2m)
        } else {
          tmpD=GenDataSEM5(n, BetaXF, indexB01, sigma2uF, indexU01, sigma2m)
        }
        X=tmpD$X; M=tmpD$M
        save(list=c("X","M","n","pM","pUF","pX"),file="CheckCode.rdata")
        
        ti=proc.time()-ti; timeSi[1]=timeSi[1]+ti[3] #####
        
        ###
        MX=NULL
        for (j in 1:pM) {
          for (i in 1:pX) {
            MX=cbind(MX, M[,j]*X[,i])
          }
        }
        M2=M^2
        
        
        ti=proc.time() #####
        
        ###############
        ###  Lasso  ###
        ###############
        
        ###  Lasso - Beta  ###
        nlambda=10; rlambda=ifelse(n>((pM-1)*pX), 0.001, 0.01)
        
        lambdaMaxi=NULL; j=1
        for (j in 1:pM) {
          Mj=M[,j]; MXj=MX[,-c((1:pX)+(j-1)*pX)]
          fitb2=EnetLm(x=MXj, y=Mj, nlambda=2)
          lambdaMaxi=c(lambdaMaxi,fitb2$fit$lambda[1])
        }
        lambdami=max(lambdaMaxi)
        lambdaS=lambdami*(rlambda)^(c(0:(nlambda-1))/(nlambda-1))
        
        
        Beta2S=list()
        Beta2i=NULL; BIC2i=numeric(nlambda)
        Beta2g=NULL; BIC2g=numeric(nlambda)
        
        d1=nlambda; j=1
        for (j in 1:pM) {
          Mj=M[,j]; MXj=MX[,-c((1:pX)+(j-1)*pX)]
          
          fitb2=EnetLm(x=MXj, y=Mj, lambda=lambdaS)
          Beta2S[[j]]=fitb2$Beta
          
          tBIC2i=fitb2$RSS+log(n)*apply(fitb2$Beta!=0,2,sum)
          idi=which.min(tBIC2i); idi
          
          tBIC2g=fitb2$RSS+log(n)*apply(fitb2$Beta,2,function(x){sum(apply(matrix(x,nrow=pX)^2,2,sum)>0)*pX})
          idg=which.min(tBIC2g); idg
          
          if (length(tBIC2i)<d1) {d1=length(tBIC2i)}
          BIC2i=BIC2i[1:d1]+tBIC2i[1:d1]
          BIC2g=BIC2g[1:d1]+tBIC2g[1:d1]
          
          Beta2i=cbind(Beta2i,matrix(fitb2$Beta[,idi],nrow=pX,byrow=F))
          Beta2g=cbind(Beta2g,matrix(fitb2$Beta[,idg],nrow=pX,byrow=F))
        }
        
        idi=which.min(BIC2i); idi
        Beta2is=matrix(sapply(Beta2S,function(x){x[,idi]}),nrow=pX)
        
        idg=which.min(BIC2g); idg
        Beta2gs=matrix(sapply(Beta2S,function(x){x[,idg]}),nrow=pX)
        
        
        ###  Lasso - Sigma2U  ###
        Beta2s2=matrix(c(Beta2is),ncol=pM)
        
        Ri2=NULL
        for (j in 1:pM) {
          Mj=M[,j]; MXj=MX[,-c((1:pX)+(j-1)*pX)]
          Ri2=cbind(Ri2, (Mj-MXj%*%Beta2s2[,j])^2)
        }
        
        nlambda=10; rlambda=ifelse(n>(pM-1), 0.001, 0.01)
        
        lambdaMaxi=NULL; j=1
        for (j in 1:pM) {
          M2j=M2[,-j]; Ri2j=Ri2[,j]
          fit2i=EnetLmNNi(x=M2j, y=Ri2j, nlambda=2)
          lambdaMaxi=c(lambdaMaxi,fit2i$fit$lambda[1])
        }
        lambdami=max(lambdaMaxi)
        lambdaS=lambdami*(rlambda)^(c(0:(nlambda-1))/(nlambda-1))
        
        Sigma2S=list(); d1=nlambda
        Sigma2i=NULL; BIC2i=numeric(nlambda)
        for (j in 1:pM) {
          M2j=M2[,-j]; Ri2j=Ri2[,j]
          
          fit2i=EnetLmNNi(x=M2j, y=Ri2j, lambda=lambdaS)
          Sigma2S[[j]]=fit2i$Beta[-1,]
          
          tBICb=fit2i$RSS+log(n)*apply(fit2i$Beta!=0,2,sum)
          i1=which.min(tBICb); i1; #fit2i$Beta[,i1]
          
          if (length(tBICb)<d1) {d1=length(tBICb)}
          BIC2i=BIC2i[1:d1]+tBICb[1:d1]
          
          Sigma2i=c(Sigma2i,fit2i$Beta[-1,i1])
        }
        
        idi=which.min(BIC2i); idi
        Sigma2is=c(sapply(Sigma2S,function(x){x[,idi]}))
        
        #####################
        ###  Lasso Index  ###
        indexIis=which(apply(Beta2is^2,2,sum)>0 | Sigma2is>0)
        
        tBetai=matrix(apply(Beta2is^2,2,mean)+Sigma2is,ncol=pM)
        nb0=apply(tBetai!=0,2,sum)
        indnb0=which(nb0>floor((rnb0*n-1)/(pX+1))); indnb0
        if (length(indnb0)>0) {
          for (inb0 in indnb0) {
            scal=abs(tBetai[,inb0])
            ord=order(scal,decreasing=TRUE)
            tBetai[-ord[1:floor((rnb0*n-1)/(pX+1))],inb0]=0
          }
        }
        indexIit=which(tBetai>0)
        ti=proc.time()-ti; timeSi[2]=timeSi[2]+ti[3] #####
        
        
        ti=proc.time() #####
        
        
        ###############
        ###  htDAG  ###
        ###############
        
        betai=matrix(0,nrow=pX,ncol=pUF); betai[,indexIit]=0.5
        sigma2ui=numeric(pUF); sigma2ui[indexIit]=0.5
        sigma2mi=rep(0.5,pM)
        
        fiti=mSEMCT12b(M, X, pUF+1,thresh,maxit,inThresh,inMaxit,thresh2,maxit2,betai,sigma2mi,sigma2ui)
        BICi=fiti$BIC; BICi[fiti$iDAGS==0]=NA; #BICi
        i2=which.min(BICi); i2
        
        Betaht=matrix(fiti$BetaX[,i2],nrow=pX)
        Sigmaht=fiti$Sigma2U[,i2]
        SigmaMht=fiti$Sigma2M[,i2]
        
        ti=proc.time()-ti; timeSi[3]=timeSi[3]+ti[3] #####
        
        ti=proc.time() #####
        
        
        ############
        ###  PC  ###
        ############
        pc.fit = pc(suffStat = list(C = cor(M), n = n),
                    indepTest = gaussCItest, ## indep.test: partial correlations
                    alpha = 0.01, verbose = FALSE, p=pM)
        temi=as(pc.fit, "amat")

        
        diag(temi)=NA
        temi=c(t(temi))
        BetaPC=temi[!is.na(temi)]
        
        ti=proc.time()-ti; timeSi[4]=timeSi[4]+ti[3] #####
        
        
        ti=proc.time() #####
        
        
        ###############
        ###  PenPC  ###
        ###############
        coeff = ne.PEN(dat=M,nlambda=100,ntau=10,V=1:pM,order=F,verbose=F)
        edgeWeights = matrix(0,pM,pM)
        edgeWeights[coeff!=0|t(coeff)!=0] =1
        
        ### If PenPC cannot run
        fixedGaps = (edgeWeights == 0)
        G = !fixedGaps
        m.max = max(rowSums(G))
        
        w.upper = which(upper.tri(diag(pM)), arr.ind = T)
        ind = w.upper[G[w.upper], ]
        if (is.null(dim(ind)[1])) {
          fit.penpc = pc.fit
          caseSi=caseSi+1
        } else {
          fit.penpc  = skeletonPENstable(suffStat = list(C = cor(M), n = n), indepTest = gaussCItest, as.integer(pM), 0.01,  edgeWeights=edgeWeights, verbose=FALSE)
        }
       
        
        temi=as(fit.penpc, "amat")

        
        diag(temi)=NA
        temi=c(t(temi))
        BetaPenPC=temi[!is.na(temi)]
        
        ti=proc.time()-ti; timeSi[5]=timeSi[5]+ti[3] #####
        
        
        ###
        temi=cbind(true=c(BetaXF),pBICi=c(Beta2i),pBICis=c(Beta2is),pBICg=c(Beta2g),pBICgs=c(Beta2gs),pHT=c(Betaht))
        resultBi[[iSim]]=temi
        
        temi2=cbind(true=sigma2uF,pBICi=c(Sigma2i),pBICis=c(Sigma2is),pHT=c(Sigmaht))
        resultUi[[iSim]]=temi2
        
        temi3=cbind(true=apply(BetaXF^2,2,sum),PC=BetaPC,PenPC=BetaPenPC)
        resultPCi[[iSim]]=temi3
        
        temi4=cbind(true=sigma2m,pHT=SigmaMht)
        resultMi[[iSim]]=temi4
      }
      
      resultBS[[iCase]]=resultBi
      resultUS[[iCase]]=resultUi
      resultPCS[[iCase]]=resultPCi
      resultMS[[iCase]]=resultMi
      resultTS[[iCase]]=timeSi
      resultCase=c(resultCase, caseSi)
      iCase=iCase+1
    }
  }
  # save(list=c("resultBS","resultUS","resultPCS","resultMS","resultTS"),file=paste0("SimResults/Sim",iset,"_htDAG_170420.rdata"))
  #save(list=c("resultBS","resultUS","resultPCS","resultMS","resultTS","resultCase"),file=paste0("SimResults/cSim",iset,"_htDAG_170420.rdata"))
}
Iset
resultTS



##########################
###  Sim 1-5:Settings  ###
##########################

Ns=c(200,500,1000); pMs=c(20,50,100); #pMs=c(10,20)
indexG00=matrix(c(1,3,1,2,2,3,1,4,4,5,6,7,7,8,8,9,8,10,11,12,12,13,14,15),byrow=TRUE,ncol=2)

resultBS=list(); resultUS=list(); resultBS5=list()
iset=1; ipM=1
for (iset in 1:5) {
  
  cat("\n");cat(iset)
  
  ###  Parameters  ###
  pM=pMs[ipM];
  pU=(pM-1)*pM/2; pUF=pM*(pM-1)
  
  index0=matrix(0,nrow=pM,ncol=pM)
  for (j in 1:nrow(indexG00)) {index0[indexG00[j,1],indexG00[j,2]]=1}
  index0i=index0; diag(index0i)=NA; index0i=c(t(index0i)); index0i=index0i[!is.na(index0i)]
  index01=index0i; index01[index01==1][c(2,4,6,8)]=0
  index02=index0i; index02[index02==1][c(5,7,10,12)]=0
  
  indexB0i=numeric(pUF); indexU0i=numeric(pUF)
  if (iset %in% c(1,3,5)) {indexB0i[index0i==1]=1}
  if (iset %in% c(2,3)) {indexU0i[index0i==1]=1}
  if (iset == 4) {indexB0i[index01==1]=1; indexU0i[index02==1]=1}
  
  if (iset %in% c(1:4)) {pX=3}
  if (iset == 5) {pX=1}
  
  if (iset %in% c(1:4)) {
    BetaXF=matrix(0,nrow=pX,ncol=pUF)
    BetaXF[,indexB0i==1]=seq(0.5,1.5,length=pX)*(-1)^(seq(pX))
  } else if (iset == 5) {
    BetaXF=matrix(0, nrow=pX, ncol=pUF)
    BetaXF[1,indexB0i==1]=rep(-1,1,length=sum(indexB0i==1))*(-1)^(seq(sum(indexB0i==1)))
  }
  
  sigma2uF=rep(0, length=pUF)
  sigma2uF[indexU0i==1]=0.5
  
  sigma2m=rep(c(1,0.5), length=pM)
  
  indexB0=indexB0i
  indexU0=indexU0i
  
  indexB01=which(indexB0!=0)
  indexU01=which(indexU0!=0)
  
  indexF=NULL; sr=1
  for (s in 1:pM) {
    for (r in 1:pM) {
      if (r != s) {
        indexF=rbind(indexF, c(s=s,r=r,sr=sr))
        sr=sr+1
      }
    }
  }
  
  resultBS[[iset]]=indexF[which(indexB0!=0),]
  resultUS[[iset]]=indexF[which(indexU0!=0),]
  
  if (iset==5) {
    resultBS5[[1]]=indexF[which(BetaXF[1,]>0),]
    resultBS5[[2]]=indexF[which(BetaXF[1,]<0),]
  }
}

sapply(resultBS,function(x){
  paste0(paste0("(",x[,1],",",x[,2],")"),collapse=", ")
})
sapply(resultBS,nrow)


sapply(resultBS5,function(x){
  paste0(paste0("(",x[,1],",",x[,2],")"),collapse=", ")
})


sapply(resultUS,function(x){
  paste0(paste0("(",x[,1],",",x[,2],")"),collapse=", ")
})


################
###  Report  ###
################

Ns=c(200,500,1000); pMs=c(20,50,100); 

indexG00=matrix(c(1,3,1,2,2,3,1,4,4,5,6,7,7,8,8,9,8,10,11,12,12,13,14,15),byrow=TRUE,ncol=2)


Iset=1:5; iset=Iset[1]
for (iset in Iset) {
  cat(iset)
  load(paste0("SimResults/cSim",iset,"_htDAG_170420.rdata"))
  
  ## Time
  tt=t(do.call("rbind",resultTS))
  
  output=NULL; i=1
  for (i in 1:nrow(tt)) {
    outputi=rbind(matrix(tt[i,],byrow=T,nrow=3))
    output=cbind(output, NA, outputi)
  }
  write.csv(output,paste0("R_tmp_Time",iset,".csv"),na="")
  
  
  VSbetatp=NULL; VSbetafp=NULL
  VSgtp=NULL; VSgfp=NULL
  VSutp=NULL; VSufp=NULL
  
  SSEb=NULL; SSEu=NULL; SSEm=NULL
  
  iCase=1; iN=1; ipM=1
  for (iN in 1:length(Ns)) {
    for (ipM in 1:length(pMs)) {
      
      ###  Parameters  ###
      n=Ns[iN]; pM=pMs[ipM];
      pU=(pM-1)*pM/2; pUF=pM*(pM-1)
      
      index0=matrix(0,nrow=pM,ncol=pM)
      for (j in 1:nrow(indexG00)) {index0[indexG00[j,1],indexG00[j,2]]=1}
      index0i=index0; diag(index0i)=NA; index0i=c(t(index0i)); index0i=index0i[!is.na(index0i)]
      index01=index0i; index01[index01==1][c(2,4,6,8)]=0
      index02=index0i; index02[index02==1][c(5,7,10,12)]=0
      
      indexB0i=numeric(pUF); indexU0i=numeric(pUF)
      if (iset %in% c(1,3,5)) {indexB0i[index0i==1]=1}
      if (iset %in% c(2,3)) {indexU0i[index0i==1]=1}
      if (iset == 4) {indexB0i[index01==1]=1; indexU0i[index02==1]=1}
      
      if (iset %in% c(1:4)) {pX=3}
      if (iset == 5) {pX=1}
      
      if (iset %in% c(1:4)) {
        BetaXF=matrix(0,nrow=pX,ncol=pUF)
        BetaXF[,indexB0i==1]=seq(0.5,1.5,length=pX)*(-1)^(seq(pX))
      } else if (iset == 5) {
        BetaXF=matrix(0, nrow=pX, ncol=pUF)
        BetaXF[1,indexB0i==1]=rep(-1,1,length=sum(indexB0i==1))*(-1)^(seq(sum(indexB0i==1)))
      }
      
      sigma2uF=rep(0, length=pUF)
      sigma2uF[indexU0i==1]=0.5
      
      sigma2m=rep(c(1,0.5), length=pM)
      
      indexB0=indexB0i
      indexU0=indexU0i
      
      indexB01=which(indexB0!=0)
      indexU01=which(indexU0!=0)
      
      indexF=NULL; sr=1
      for (s in 1:pM) {
        for (r in 1:pM) {
          if (r != s) {
            indexF=rbind(indexF, c(s=s,r=r,sr=sr))
            sr=sr+1
          }
        }
      }
      
      
      ###  Summary - VS  ###
      ## Beta
      temi=sapply(resultBS[[iCase]],function(x) {apply(x,2,function(y){sum(apply(matrix(y,nrow=pX)^2,2,sum)>0 & apply(matrix(x[,1],nrow=pX)^2,2,sum)>0)})})
      VSbetatp=cbind(VSbetatp, apply(temi,1,mean))
      temi=sapply(resultBS[[iCase]],function(x) {apply(x,2,function(y){sum(apply(matrix(y,nrow=pX)^2,2,sum)>0 & apply(matrix(x[,1],nrow=pX)^2,2,sum)==0)})})
      VSbetafp=cbind(VSbetafp, apply(temi,1,mean))
      # indexB0
      
      
      ## Sigma2U
      temi=sapply(resultUS[[iCase]],function(x) {apply(x[,1]!=0 & x!=0,2,sum)})
      VSutp=cbind(VSutp, apply(temi,1,mean))
      temi=sapply(resultUS[[iCase]],function(x) {apply(x[,1]==0 & x!=0,2,sum)})
      VSufp=cbind(VSufp, apply(temi,1,mean))
      # indexU0
      
      
      ## Group
      index2=matrix(c(1,1,2,2,3,3,6,4),byrow=TRUE,ncol=2)
      net0=c((apply(BetaXF^2,2,mean)+sigma2uF)>0)
      
      s=5; tpi=NULL; fpi=NULL
      for (s in 1:nrow(index2)) {
        s1=index2[s,1]; s2=index2[s,2]
        temi=(sapply(resultBS[[iCase]],function(x){apply(matrix(x[,s1],nrow=pX)^2,2,sum)>0}) | sapply(resultUS[[iCase]],function(x){x[,s2]!=0}))
        tpi=c(tpi,mean(apply(net0 & temi,2,sum)))
        fpi=c(fpi,mean(apply(!net0 & temi,2,sum)))
      }
      names(tpi)=c("true","pBICi","pBICis","pHT")
      names(fpi)=c("true","pBICi","pBICis","pHT")
      
      ## PC & PenPC
      
      temi=sapply(resultPCS[[iCase]],function(x) {apply(net0!=0 & x!=0,2,sum)})
      tpi=c(tpi, apply(temi,1,mean)[2:3])

      temi2=sapply(resultPCS[[iCase]],function(x) {apply(net0==0 & x!=0,2,sum)})
      fpi=c(fpi, apply(temi2,1,mean)[2:3])
      
      
      VSgtp=cbind(VSgtp, tpi)
      VSgfp=cbind(VSgfp, fpi)
      
      
      ###  Summary - SSE  ###
     
      temi=apply(sapply(resultBS[[iCase]],function(x){sqrt(apply((x[,c(1,6)]-x[,1])^2,2,sum))}),1,mean)
      temi2=apply(sapply(resultUS[[iCase]],function(x){sqrt(apply((x[,c(1,4)]-x[,1])^2,2,sum))}),1,mean)
      temi3=apply(sapply(resultMS[[iCase]],function(x){sqrt(apply((x-x[,1])^2,2,sum))}),1,mean)
      
      SSEb=cbind(SSEb,temi)
      SSEu=cbind(SSEu,temi2)
      SSEm=cbind(SSEm,temi3)
      
      iCase=iCase+1
    }
  }
  
  ## VS
  output=NULL; i=1
  for (i in 1:nrow(VSgtp)) {
    outputi=rbind(matrix(VSgtp[i,],byrow=T,nrow=3), matrix(VSgfp[i,],byrow=T,nrow=3))
    output=cbind(output, NA, outputi)
  }
  write.csv(output,paste0("R_tmp_VS",iset,".csv"),na="")
  
  ## SSE
  output=NULL; i=1
  for (i in 1:nrow(SSEb)) {
    outputi=cbind(matrix(SSEb[i,],byrow=T,nrow=3),NA,matrix(SSEu[i,],byrow=T,nrow=3),NA,matrix(SSEm[i,],byrow=T,nrow=3))
    output=cbind(output, NA, NA, outputi)
  }
  write.csv(output,paste0("R_tmp_SSE",iset,".csv"),na="")
}


### Combine output ###
library(xlsx)
path0="../Results/Temp.xlsx"

output=NULL
for (iset in Iset) {
  outputi=read.csv(paste0("R_tmp_VS",iset,".csv"),na="")
  output=rbind(output,NA,outputi)
}
write.xlsx(output,path0,row.names=T,sheetName="VS",showNA=FALSE)


output=NULL
for (iset in Iset) {
  outputi=read.csv(paste0("R_tmp_SSE",iset,".csv"),na="")
  output=rbind(output,NA,outputi)
}
write.xlsx(output,path0,row.names=T,sheetName="SSE",append=TRUE,showNA=FALSE)


output=NULL
for (iset in Iset) {
  outputi=read.csv(paste0("R_tmp_Time",iset,".csv"),na="")
  output=rbind(output,NA,outputi)
}
write.xlsx(output,path0,row.names=T,sheetName="Time",append=TRUE,showNA=FALSE)




