// [[Rcpp::depends(RcppEigen)]]
// [[Rcpp::interfaces(r,cpp)]]
#include <RcppEigen.h>
#include <Rcpp.h>
#include <ctime>

using namespace Rcpp;


//////////////////////////////////////////////
/////   Test 12:SEM with mixed effects   /////
//////////////////////////////////////////////

// [[Rcpp::export]]
List mSEMCT12b(Eigen::MatrixXd M, Eigen::MatrixXd X,
               int nTrim, double thresh, int maxit, double inThresh, int inMaxit,
               double thresh2, int maxit2,
               Eigen::MatrixXd BetaX, Eigen::VectorXd Sigma2M, Eigen::VectorXd Sigma2U) {
  
  int N=M.rows(), pM=M.cols(), pUF=pM*(pM-1), pX=X.cols();
  int j, s, r, sr, it, s1, r1, j1, it2;
  
  int nA=0, nSM=0;
  Eigen::MatrixXi indexAct=Eigen::MatrixXi::Zero(pM,pM);
  Eigen::VectorXi nnA=Eigen::VectorXi::Zero(pM);
  
  // Eigen::VectorXi orderi(pM);
  
  Eigen::MatrixXd M2(N, pM), XM(N,pX*pM), XMi(N,pX*(pM-1));
  Eigen::VectorXd Mi(N), f1(N), f2(N);
  
  Eigen::VectorXd BetaXi(pX*(pM-1));
  Eigen::VectorXd Sigma2Mi(pM), Sigma2Ui=Eigen::VectorXd::Zero(pUF);
  Eigen::MatrixXd BetaXS=Eigen::MatrixXd::Zero(pUF*pX, pUF+1), Sigma2MS(pM, pUF+1), Sigma2US=Eigen::MatrixXd::Zero(pUF, pUF+1);
  
  Eigen::MatrixXd Ri(N,pM), Wi=Eigen::MatrixXd::Zero(N,pM);
  
  double thetai, thetad, threshS2, threshi;
  
  int iTrim;
  
  Eigen::MatrixXi Mnet=Eigen::MatrixXi::Zero(pM, pM);
  
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(pUF+1), nIte=Eigen::VectorXi::Zero(pUF+1), iDAGS=Eigen::VectorXi::Zero(pUF+1);
  int iDAG=0;
  Eigen::VectorXd llS=Eigen::VectorXd::Zero(pUF+1);
  Eigen::VectorXd AICS=Eigen::VectorXd::Zero(pUF+1), BICS=Eigen::VectorXd::Zero(pUF+1);
  
  Eigen::VectorXd scale(2), ratio(2);
  double ratio0;
  int s0=0, r0=0, sr0=0;
  int iTwo=0;
  
  ///  Initial Constant  ///
  
  // Interaction X*M, M2
  sr=0;
  for (s=0; s<pM; ++s) {
    for (j=0; j<pX; ++j) {
      XM.col(sr)=M.col(s).array()*X.col(j).array();
      ++sr;
    }
    M2.col(s)=M.col(s).array()*M.col(s).array();
  }
  
  
  ///  Initial Parameters  ///
  
  // Initial - indexAct
  sr=0;
  for (s=0; s<pM; ++s) {
    for (r=0; r<pM; ++r) {
      if (r != s) {
        
        if (BetaX.col(sr).squaredNorm()>0 || Sigma2U(sr)>0) {
          ++nnA(s);
          indexAct(s,r)=sr+1;
          ++nA;
        }
        
        ++sr;
      }
    }
  }
  
  // Initial - Symmetric
  for (s=0; s<(pM-1); ++s) {
    for (r=s+1; r<pM; ++r) {
      if (indexAct(s,r)>0 && indexAct(r,s)>0) {
        ++nSM;
      }
    }
  }
  
  // Initial - Residual & Weights
  for (s=0; s<pM; ++s) {
    Wi.col(s)=Wi.col(s).array()+Sigma2M(s);
  }
  Ri=M;
  for (s=0; s<pM; ++s) {
    for (r=0; r<pM; ++r) {
      if (indexAct(s,r)>0) {
        sr=indexAct(s,r)-1;
        Ri.col(s)=Ri.col(s).array()-(X*BetaX.col(sr)).array()*(M.col(r).array());
        Wi.col(s)=Wi.col(s).array()+Sigma2U(sr)*(M2.col(r).array());
      }
    }
  }
  
  
  ///  Update Estimates with Half cut  ///
  iTrim=0;
  EstH:;
  for (s=0; s<pM; ++s) {
    Est:;
    // Rcout << " s=" << s << std::endl;
    
    it=0;
    while (1) {
      ++it; threshi=0.0;
      
      // Update - BetaX - QR
      if (nnA(s)>0) {
        
        // extract
        j=0;
        for (r=0; r<pM; ++r) {
          if (indexAct(s,r)>0) {
            for (j1=0; j1<pX; ++j1) {
              XMi.col(j*pX+j1)=XM.col(r*pX+j1).array()/Wi.col(s).array().sqrt();
            }
            ++j;
          }
        }
        Mi=M.col(s).array()/Wi.col(s).array().sqrt();
        
        // estimate
        BetaXi=XMi.block(0,0,N,nnA(s)*pX).colPivHouseholderQr().solve(Mi);
        
        // record
        j=0;
        for (r=0; r<pM; ++r) {
          if (indexAct(s,r)>0) {
            for (j1=0; j1<pX; ++j1) {
              sr=indexAct(s,r)-1;
              thetad=BetaXi(j*pX+j1)-BetaX(j1,sr);
              Ri.col(s)=Ri.col(s).array()-thetad*XM.col(r*pX+j1).array();
              threshi+=std::abs(thetad);
              BetaX(j1,sr)=BetaXi(j*pX+j1);
            }
            ++j;
          }
        }
        
        // Ri2=Ri.col(s)*Ri.col(s); 
      } // Update - BetaX - sth M
      
      
      // Update - Sigma2M Sigma2U - Coordinate Newton-Raphson
      it2=0; Sigma2Mi=Sigma2M; Sigma2Ui=Sigma2U;
      while (1) {
        ++it2;
        threshS2=0.0;
        
        // Update - Sigma2M
        f1=1.0/Wi.col(s).array()-Ri.col(s).array()*Ri.col(s).array()/Wi.col(s).array()/Wi.col(s).array();
        f2=2.0*Ri.col(s).array()*Ri.col(s).array()/Wi.col(s).array()/Wi.col(s).array()/Wi.col(s).array()-1.0/Wi.col(s).array()/Wi.col(s).array();
        
        thetai=Sigma2M(s)-f1.sum()/f2.sum();
        if (thetai < 0.0 || f2.sum() < 0.0) {
          thetai=0.0001;
        }
        thetad=thetai-Sigma2M(s);
        Wi.col(s)=Wi.col(s).array()+thetad;
        Sigma2M(s)=thetai; threshS2+=std::abs(thetad);
        
        // Update - Sigma2U
        if (nnA(s)>0) {
          for (r=0; r<pM; ++r) {
            if (indexAct(s,r)>0) {
              sr=indexAct(s,r)-1;
              
              f1=(1.0/Wi.col(s).array()-Ri.col(s).array()*Ri.col(s).array()/Wi.col(s).array()/Wi.col(s).array())*M2.col(r).array();
              f2=(2.0*Ri.col(s).array()*Ri.col(s).array()/Wi.col(s).array()/Wi.col(s).array()/Wi.col(s).array()-1.0/Wi.col(s).array()/Wi.col(s).array())*M2.col(r).array()*M2.col(r).array();
              
              thetai=Sigma2U(sr)-f1.sum()/f2.sum();
              if (thetai < 0.0 || f2.sum() < 0.0) {
                thetai=0.0;
              }
              thetad=thetai-Sigma2U(sr);
              Wi.col(s)=Wi.col(s).array()+thetad*(M2.col(r).array());
              Sigma2U(sr)=thetai; threshS2+=std::abs(thetad);
              
            }
          }
        }
        
        if (threshS2 < inThresh) break;
        if (it2 >= inMaxit) break;
        
      } // Update - Sigma2M Sigma2U - sth M
      
      threshi+=std::abs(Sigma2Mi(s)-Sigma2M(s)); 
      if (nnA(s)>0) {
        for (r=0; r<pM; ++r) {
          if (indexAct(s,r)>0) {
            sr=indexAct(s,r)-1;
            threshi+=std::abs(Sigma2Ui(sr)-Sigma2U(sr)); 
          }
        }
      }
      
      // if (nTrim > 0) {
      if (nSM>0) {
        if (threshi < thresh2) {goto NextM;}
        if (it >= maxit2) {++flag(iTrim); goto NextM;}
      } else {
        if (threshi < thresh) {goto NextM;}
        if (it >= maxit) {++flag(iTrim); goto NextM;}
      }
      
    } // Update - sth M
    
    NextM:;
    nIte(iTrim)+=it;
    if (iTwo == 0) {
      if (iTrim > 0) break;
    }
  }
  if (iTwo == 1) {
    iTwo=0;
  }
  
  
  ///  DAG - Check  ///
  if (iDAG==0 && nSM<=0) {
    
    Mnet.setZero(pM,pM);
    for (s=0; s<pM; ++s) {
      for (r=0; r<pM; ++r) {
        
        if (indexAct(s,r) > 0.5) {
          // Rcout << " s=" << s <<" r=" << r << " sr=" << indexAct(s,r) << std::endl;
          Mnet(s,r)=1;
          
          // r parent
          for (j1=0; j1<pM; ++j1) {
            if (Mnet(r,j1)==1) {
              Mnet(s,j1)=1;
            }
          }
          // s child
          for (j1=0; j1<pM; ++j1) {
            if (Mnet(j1,s)==1) {
              for (r1=0; r1<pM; ++r1) {
                if (Mnet(s,r1)==1) {
                  Mnet(j1,r1)=1;
                }
              }
            }
          }
        }
        
      }
    }
    
    iDAG=1;
    for (j=0; j<pM; ++j) {
      if (Mnet(j,j)==1) {
        iDAG=0;
        break;
      }
    }
  }
  
  // if DAG, then log-likelihood, AIC and BIC
  if (iDAG == 1) {
    for (s=0; s<pM; ++s) {
      llS(iTrim)+=(Ri.col(s).array()*Ri.col(s).array()/Wi.col(s).array()+log(Wi.col(s).array())).sum();
    }
    
    for (j=0, j1=0; j<pUF; ++j) {
      if (Sigma2U(j) > 0.0) {
        ++j1;
      }
    }
    AICS(iTrim)=llS(iTrim)+2.0*(j1+nA*pX+pM);
    BICS(iTrim)=llS(iTrim)+log(N)*(j1+nA*pX+pM);
  }
  
  
  ///  Record results  ///
  iDAGS(iTrim)=iDAG;
  
  BetaXS.col(iTrim)=BetaX;
  Sigma2MS.col(iTrim)=Sigma2M;
  Sigma2US.col(iTrim)=Sigma2U;
  ++iTrim;
  
  if (nTrim == 0) goto Exit;
  if (iTrim > nTrim) goto Exit;
  if (nA == 0) goto Exit;
  
  
  ///  Half cut: One Out at a time - keep either jk or kj based on the size  ///
  if (nSM > 0) {
    ratio0=0.5;
    for (s1=0; s1<(pM-1); ++s1) {
      for (r1=s1+1; r1<pM; ++r1) {
        if (indexAct(s1,r1)>0 && indexAct(r1,s1)>0) {
          
          scale(0)=BetaX.col(indexAct(s1,r1)-1).squaredNorm()/pX+Sigma2U(indexAct(s1,r1)-1);
          scale(1)=BetaX.col(indexAct(r1,s1)-1).squaredNorm()/pX+Sigma2U(indexAct(r1,s1)-1);
          
          ratio(0)=scale(0)/scale(1);
          ratio(1)=scale(1)/scale(0);
          
          if (ratio(0)>ratio(1)) {
            if (ratio(0)>ratio0) {
              ratio0=ratio(0);
              s0=r1; r0=s1; sr0=indexAct(r1,s1)-1;
            }
          } else {
            if (ratio(1)>ratio0) {
              ratio0=ratio(1);
              s0=s1; r0=r1; sr0=indexAct(s1,r1)-1;
            }
          }
          
        }
      }
    } // Half cut
    
    // Update active set
    s=s0; r=r0; sr=sr0;
    
    indexAct(s,r)=0;
    --nnA(s); --nA;
    
    for (r1=0; r1<pX; ++r1) {
      Ri.col(s)=Ri.col(s).array()+BetaX(r1,sr)*XM.col(r*pX+r1).array();
      BetaX(r1,sr)=0.0;
    }
    
    Wi.col(s)=Wi.col(s).array()-Sigma2U(sr)*(M2.col(r).array());
    Sigma2U(sr)=0.0;
    
    --nSM;
    if (nSM>0) {
      goto Est;
    } else {
      iTwo=1;
      goto EstH;
    }
  }
  
  
  ///  Trim: One Out at a time  ///
  if (nA > 0) {
    thetad=0.0; s1=0; r1=0;
    for (s1=0; s1<pM; ++s1) {
      for (r1=0; r1<pM; ++r1) {
        if (indexAct(s1,r1) > 0.5) {
          
          thetai=BetaX.col(indexAct(s1,r1)-1).squaredNorm()/pX+Sigma2U(indexAct(s1,r1)-1);
          
          if (thetad == 0.0) {
            thetad=thetai; s0=s1; r0=r1;
          } else if (thetad > 0.0 && thetai < thetad) {
            thetad=thetai; s0=s1; r0=r1;
          }
          
        }
        
      }
    }
    
    // Update active set
    s=s0; r=r0; sr=indexAct(s0,r0)-1;
    
    indexAct(s,r)=0;
    --nnA(s); --nA;
    
    for (r1=0; r1<pX; ++r1) {
      Ri.col(s)=Ri.col(s).array()+BetaX(r1,sr)*XM.col(r*pX+r1).array();
      BetaX(r1,sr)=0.0;
    }
    
    Wi.col(s)=Wi.col(s).array()-Sigma2U(sr)*(M2.col(r).array());
    Sigma2U(sr)=0.0;
    
    
    if (nA>=0) {goto Est;}
  } // Trim: One Out
  
  
  Exit:;
  
  return List::create(//Named("BetaX")=BetaX, Named("Sigma2M")=Sigma2M, Named("Sigma2U")=Sigma2U,
    Named("BetaX")=BetaXS.block(0,0,pUF*pX,iTrim),
    Named("Sigma2M")=Sigma2MS.block(0,0,pM,iTrim),
    Named("Sigma2U")=Sigma2US.block(0,0,pUF,iTrim),
    Named("nIte")=nIte.head(iTrim), Named("Flag")=flag.head(iTrim),
    Named("ll")=llS.head(iTrim), Named("AIC")=AICS.head(iTrim), Named("BIC")=BICS.head(iTrim),
    Named("nA")=nA, Named("nSM")=nSM, Named("indexAct")=indexAct,
    Named("iDAGS")=iDAGS.head(iTrim));
}




